/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package com.netease.nimflutter.services

import android.content.Context
import com.netease.nimflutter.*
import com.netease.nimlib.sdk.NIMClient
import com.netease.nimlib.sdk.mixpush.MixPushService
import com.netease.nimlib.sdk.settings.SettingsService
import com.netease.nimlib.sdk.settings.model.NoDisturbConfig
import kotlinx.coroutines.suspendCancellableCoroutine

class FLTSettingsService(
    applicationContext: Context,
    nimCore: NimCore,
) : FLTService(applicationContext, nimCore) {

    override val serviceName = "SettingsService"

    init {
        registerFlutterMethodCalls(
            "enableMobilePushWhenPCOnline" to ::enableMobilePushWhenPCOnline,
            "isMobilePushEnabledWhenPCOnline" to ::isMobilePushEnabledWhenPCOnline,
            "enableNotification" to ::enableNotification,
            "enablePushService" to ::enablePushService,
            "isPushServiceEnabled" to ::isPushServiceEnabled,
            "getPushNoDisturbConfig" to ::getPushNoDisturbConfig,
            "setPushNoDisturbConfig" to ::setPushNoDisturbConfig,
            "isPushShowDetailEnabled" to ::isPushShowDetailEnabled,
            "enablePushShowDetail" to ::enablePushShowDetail,
            "updateNotificationConfig" to ::updateNotificationConfig,
        )
    }

    private suspend fun enableMobilePushWhenPCOnline(arguments: Map<String, *>): NimResult<Nothing> {
        return suspendCancellableCoroutine { cont ->
            val enableMobilePushWhenPCOnline: Boolean by arguments
            NIMClient.getService(SettingsService::class.java)
                .updateMultiportPushConfig(enableMobilePushWhenPCOnline.not())
                .setCallback(NimResultContinuationCallbackOfNothing(cont))
        }
    }

    private suspend fun isMobilePushEnabledWhenPCOnline(arguments: Map<String, *>): NimResult<Boolean> {
        val enabled = NIMClient.getService(SettingsService::class.java).isMultiportPushOpen
        return NimResult(
            code = 0,
            data = enabled.not()
        )
    }

    private suspend fun enableNotification(arguments: Map<String, *>): NimResult<Nothing> {
        val enableRegularNotification: Boolean by arguments
        val enableRevokeMessageNotification: Boolean by arguments
        NIMClient.toggleNotification(enableRegularNotification)
        NIMClient.toggleRevokeMessageNotification(enableRevokeMessageNotification)
        return NimResult.SUCCESS
    }

    private suspend fun enablePushService(arguments: Map<String, *>): NimResult<Nothing> {
        return suspendCancellableCoroutine { cont ->
            val enable: Boolean by arguments
            NIMClient.getService(MixPushService::class.java)
                .enable(enable)
                .setCallback(NimResultContinuationCallbackOfNothing(cont))
        }
    }

    private suspend fun isPushServiceEnabled(arguments: Map<String, *>): NimResult<Boolean> {
        return NimResult(
            code = 0,
            data = NIMClient.getService(MixPushService::class.java).isEnable
        )
    }

    private suspend fun getPushNoDisturbConfig(arguments: Map<String, *>): NimResult<NoDisturbConfig> {
        return NimResult(
            code = 0,
            data = NIMClient.getService(MixPushService::class.java).pushNoDisturbConfig,
            convert = {
                mapOf(
                    "enable" to it.isOpen,
                    "startTime" to it.startTimeString,
                    "endTime" to it.stopTimeString,
                )
            },
        )
    }

    private suspend fun setPushNoDisturbConfig(arguments: Map<String, *>): NimResult<Nothing> {
        return suspendCancellableCoroutine { cont ->
            val enable: Boolean by arguments
            val startTime: String by arguments
            val endTime: String by arguments
            NIMClient.getService(MixPushService::class.java).setPushNoDisturbConfig(
                enable, startTime, endTime
            ).setCallback(NimResultContinuationCallbackOfNothing(cont))
        }
    }

    private suspend fun enablePushShowDetail(arguments: Map<String, *>): NimResult<Nothing> {
        return suspendCancellableCoroutine { cont ->
            val enable: Boolean by arguments
            NIMClient.getService(MixPushService::class.java).setPushShowNoDetail(enable.not())
                .setCallback(NimResultContinuationCallbackOfNothing(cont))
        }
    }

    private suspend fun isPushShowDetailEnabled(arguments: Map<String, *>): NimResult<Boolean> {
        return NimResult(
            code = 0,
            data = NIMClient.getService(MixPushService::class.java).isPushShowNoDetail.not()
        )
    }

    private suspend fun updateNotificationConfig(arguments: Map<String, *>): NimResult<Nothing> {
        NIMClient.updateStatusBarNotificationConfig(convertToStatusBarNotificationConfig(arguments))
        return NimResult.SUCCESS
    }



}