/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

import Foundation
import NIMSDK

enum AudioRecorderType: String {
    case StartRecord    = "startRecord"
    case StopRecord     = "stopRecord"
    case CancelRecord   = "cancelRecord"
    case AudioRecording = "isAudioRecording"
    case Amplitude      = "getAmplitude"
}

class FLTAudioRecorderService: FLTBaseService, FLTService {
    
    let mediaManager = NIMSDK.shared().mediaManager
    
    func serviceName() -> String {
        return ServiceType.AudioRecorderService.rawValue
    }
    
    func onMethodCalled(_ method: String, _ arguments: [String : Any], _ resultCallback: ResultCallback) {
        
        switch method {
        case AudioRecorderType.StartRecord.rawValue:
            
            startAudioRecord(arguments, resultCallback)
            break
            
        case AudioRecorderType.StopRecord.rawValue:
            
            stopAudioRecord(arguments, resultCallback)
            break
            
        case AudioRecorderType.CancelRecord.rawValue:
            
            cancelAudioRecord(arguments, resultCallback)
            break
            
        case AudioRecorderType.AudioRecording.rawValue:
            
            isAudioRecord(arguments, resultCallback)
            break
            
        case AudioRecorderType.Amplitude.rawValue:
            
            getCurrentRecordAmplitude(arguments, resultCallback)
            break
            
        default:
            break
        }
        
    }
    
    func register(_ nimCore: NimCore) {
        self.nimCore = nimCore
        nimCore.addService(self)
    }
    
    func startAudioRecord(_ argument: [String : Any], _ result: ResultCallback){
        mediaManager.record(getRecordAudioType(argument), duration: getDuration(argument))
    }
    
    func stopAudioRecord(_ argument: [String : Any], _ result: ResultCallback){
        mediaManager.stopRecord()
        successCallBack(result, true)
    }
    
    func cancelAudioRecord(_ argument: [String : Any], _ result: ResultCallback){
        mediaManager.cancelRecord()
        successCallBack(result, true)
    }
    
    func isAudioRecord(_ argument: [String : Any], _ result: ResultCallback){
        let success = mediaManager.isRecording()
        successCallBack(result, success)
    }
    
    func getCurrentRecordAmplitude(_ argument: [String : Any], _ result: ResultCallback){
        let volume = Int(mediaManager.recordPeakPower())
        successCallBack(result, volume)
    }
    
}

extension FLTAudioRecorderService: NIMMediaManagerDelegate {
    
    func recordAudio(_ filePath: String?, didBeganWithError error: Error?) {
        var arguments = [String : Any]()
        arguments["filePath"] = filePath
        notifyEvent("onRecordStart",&arguments)
    }
    
    func recordAudio(_ filePath: String?, didCompletedWithError error: Error?) {
        var arguments = [String : Any]()
        arguments["filePath"] = filePath
        notifyEvent("onRecordSuccess", &arguments)
    }
    
    func recordAudioProgress(_ currentTime: TimeInterval) {
        var arguments = [String : Any]()
        notifyEvent("onRecordSuccess", &arguments)
    }
    
    func recordAudioDidCancelled() {
        var arguments = [String : Any]()
        notifyEvent("onRecordCancel", &arguments)
    }
    
    func notifyEvent(_ state: String, _ arguments: inout [String : Any]){
        arguments["serviceName"] = serviceName()
        arguments["recordState"] = state
        nimCore?.getMethodChannel()?.invokeMethod("onRecordStateChange", arguments)
    }
    

}
