// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim_core;

class ChatroomService {
  factory ChatroomService() {
    if (_singleton == null) {
      _singleton = ChatroomService._();
    }
    return _singleton!;
  }

  ChatroomService._();

  static ChatroomService? _singleton;

  ChatroomServicePlatform get _platform => ChatroomServicePlatform.instance;

  /// 加入聊天室
  Future<NIMResult<NIMChatroomEnterResult>> enterChatroom(
      NIMChatRoomEnterRequest request) {
    return _platform.enterChatroom(request);
  }

  /// 退出聊天室
  Future<NIMResult<void>> exitChatroom(String roomId) {
    return _platform.exitChatroom(roomId);
  }

  /// 创建消息
  Future<NIMResult<NIMChatroomMessage>> _createMessage(
      Map<String, dynamic> arguments) {
    return _platform.createChatroomMessage(arguments);
  }

  /// 发送聊天室消息
  ///
  /// 聊天室消息通过 [ChatroomMessageBuilder] 进行创建
  ///
  /// [resend] 如果是发送失败后重发，标记为true，否则填false，默认为
  Future<NIMResult<NIMChatroomMessage>> sendChatroomMessage(
      NIMChatroomMessage message,
      [bool resend = false]) async {
    return _platform.sendChatroomMessage(message, resend);
  }

  /// 聊天室事件流
  ///
  /// [NIMChatroomStatusEvent] 聊天室状态事件
  ///
  /// [NIMChatroomKickOutEvent] 聊天室离开事件
  Stream<NIMChatroomEvent> get onEventNotified {
    return _platform.onEventNotified;
  }

  /// 聊天室消息
  ///
  Stream<List<NIMChatroomMessage>> get onMessageReceived {
    return _platform.onMessageReceived;
  }

  /// 聊天室消息状态变更
  ///
  Stream<NIMChatroomMessage> get onMessageStatusChanged =>
      _platform.onMessageStatusChanged;

  /// 聊天室消息附件上传/下载进度通知，以 [NIMMessage.uuid] 作为key
  ///
  Stream<NIMAttachmentProgress> get onMessageAttachmentProgressUpdate =>
      _platform.onMessageAttachmentProgressUpdate;

  /// 下载聊天室消息附件
  Future<NIMResult<void>> downloadAttachment(NIMChatroomMessage message,
      [bool thumb = false]) {
    return _platform.downloadAttachment(message, thumb);
  }

  /// 获取历史消息,可选择给定时间往前或者往后查询，若方向往前，则结果排序按时间逆序，反之则结果排序按时间顺序。
  ///
  /// [roomId]    聊天室id <p>
  /// [startTime] 时间戳，单位毫秒 <p>
  /// [limit]     可拉取的消息数量 <p>
  /// [direction] 查询方向 <p>
  /// [messageTypeList] 查询的消息类型
  Future<NIMResult<List<NIMChatroomMessage>>> fetchMessageHistory({
    required String roomId,
    required int startTime,
    required int limit,
    required QueryDirection direction,
    List<NIMMessageType>? messageTypeList,
  }) {
    return _platform.fetchMessageHistory(
      roomId: roomId,
      startTime: startTime,
      limit: limit,
      direction: direction,
      messageTypeList: messageTypeList,
    );
  }

  /// 获取当前聊天室信息
  Future<NIMResult<NIMChatroomInfo>> fetchChatroomInfo(String roomId) {
    return _platform.fetchChatroomInfo(roomId);
  }

  /// 更新聊天室信息
  Future<NIMResult<void>> updateChatroomInfo({
    required String roomId,
    required NIMChatroomUpdateRequest request,
    bool needNotify = true,
    Map<String, Object>? notifyExtension,
  }) {
    return _platform.updateChatroomInfo(
      roomId: roomId,
      request: request,
      needNotify: needNotify,
      notifyExtension: notifyExtension,
    );
  }

  /// 获取当前聊天室成员
  ///
  /// [roomId]    聊天室id <p>
  /// [startTime] 时间戳，单位毫秒, 设置为 0 会使用当前服务器时间 <p>
  /// [limit]     可拉取的消息数量 <p>
  /// [queryType] 查询的类型, [NIMChatroomMemberQueryType]
  Future<NIMResult<List<NIMChatroomMember>>> fetchChatroomMembers({
    required String roomId,
    required NIMChatroomMemberQueryType queryType,
    required int limit,
    int startTime = 0,
  }) {
    return _platform.fetchChatroomMembers(
      roomId: roomId,
      queryType: queryType,
      limit: limit,
      startTime: startTime,
    );
  }

  /// 获取当前聊天室成员
  ///
  /// [roomId]      聊天室id <p>
  /// [accountList] 成员账号列表 <p>
  Future<NIMResult<List<NIMChatroomMember>>> fetchChatroomMembersByAccount({
    required String roomId,
    required List<String> accountList,
  }) async {
    return _platform.fetchChatroomMembersByAccount(
        roomId: roomId, accountList: accountList);
  }

  /// 更新聊天室内的自身信息
  /// [roomId]               聊天室id
  /// [request]              可更新的本人角色信息
  /// [needNotify]           是否通知
  /// [notifyExtension]      更新聊天室信息扩展字段，这个字段会放到更新聊天室信息通知的扩展字段中
  Future<NIMResult<void>> updateChatroomMyMemberInfo({
    required String roomId,
    required NIMChatroomUpdateMyMemberInfoRequest request,
    bool needNotify = true,
    Map<String, dynamic>? notifyExtension,
  }) async {
    return _platform.updateChatroomMyMemberInfo(
      roomId: roomId,
      request: request,
      needNotify: needNotify,
      notifyExtension: notifyExtension,
    );
  }

  ///
  /// 设置/取消设置聊天室管理员
  ///
  /// [isAdd]        true:设置, false:取消设置
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  Future<NIMResult<NIMChatroomMember>> markChatroomMemberBeManager({
    required bool isAdd,
    required NIMChatroomMemberOptions options,
  }) {
    return _platform.markChatroomMemberBeManager(
        isAdd: isAdd, options: options);
  }

  ///
  /// 设置/取消设置聊天室普通成员
  ///
  /// [isAdd]        true:设置, false:取消设置
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  Future<NIMResult<NIMChatroomMember>> markChatroomMemberBeNormal({
    required bool isAdd,
    required NIMChatroomMemberOptions options,
  }) {
    return _platform.markChatroomMemberBeNormal(isAdd: isAdd, options: options);
  }

  ///
  /// 踢掉聊天室特定成员
  ///
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  Future<NIMResult<void>> kickChatroomMember(NIMChatroomMemberOptions options) {
    return _platform.kickChatroomMember(options);
  }

  ///
  /// 添加/移出聊天室黑名单
  ///
  /// [isAdd]        true:设置, false:取消设置
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  Future<NIMResult<NIMChatroomMember>> markChatroomMemberInBlackList({
    required bool isAdd,
    required NIMChatroomMemberOptions options,
  }) {
    return _platform.markChatroomMemberInBlackList(
        isAdd: isAdd, options: options);
  }

  ///
  /// 添加/解除禁言聊天室成员
  ///
  /// [isAdd]        true:设置, false:取消设置
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  Future<NIMResult<NIMChatroomMember>> markChatroomMemberMuted({
    required bool isAdd,
    required NIMChatroomMemberOptions options,
  }) {
    return _platform.markChatroomMemberMuted(isAdd: isAdd, options: options);
  }

  ///
  /// 添加/解除临时禁言聊天室成员
  ///
  /// [duration]     禁言时长，单位ms。设置为 0 会取消禁言
  /// [memberOption] 请求参数，包含聊天室id，帐号id以及可选的扩展字段
  /// [needNotify]   是否需要发送广播通知，true：通知，false：不通知
  Future<NIMResult<void>> markChatroomMemberTempMuted({
    required int duration,
    required NIMChatroomMemberOptions options,
    bool needNotify = false,
  }) {
    return _platform.markChatroomMemberTempMuted(
      duration: duration,
      options: options,
      needNotify: needNotify,
    );
  }

  /// 获取聊天室队列
  ///
  /// [roomId] 聊天室ID <p>
  Future<NIMResult<List<NIMChatroomQueueEntry>>> fetchChatroomQueue(
      String roomId) {
    return _platform.fetchChatroomQueue(roomId);
  }

  /// 更新聊天室队列
  ///
  /// [roomId] 聊天室ID <p>
  /// [entry] 要更新的队列项 <p>
  /// [isTransient] (可选参数，不传默认false)。true表示当提交这个新元素的用户从聊天室掉线或退出的时候，需要删除这个元素；默认false表示不删除
  Future<NIMResult<void>> updateChatroomQueueEntry({
    required String roomId,
    required NIMChatroomQueueEntry entry,
    bool isTransient = false,
  }) {
    return _platform.updateChatroomQueueEntry(
      roomId: roomId,
      entry: entry,
      isTransient: isTransient,
    );
  }

  /// 批量更新聊天室队列
  ///
  /// [roomId]          聊天室ID <p>
  /// [entry]           要更新的队列项列表 <p>
  /// [needNotify]      是否需要发送广播通知 <p>
  /// [notifyExtension] 通知中的自定义字段 <p>
  /// 返回不在队列中的元素列表
  Future<NIMResult<List<String>>> batchUpdateChatroomQueue({
    required String roomId,
    required List<NIMChatroomQueueEntry> entryList,
    bool needNotify = true,
    Map<String, dynamic>? notifyExtension,
  }) {
    return _platform.batchUpdateChatroomQueue(
      roomId: roomId,
      entryList: entryList,
      needNotify: needNotify,
      notifyExtension: notifyExtension,
    );
  }

  /// 从列表中删除某个元素
  ///
  /// [roomId] 聊天室ID <p>
  /// [key] 要删除的key，null表示移除队头元素 <p>
  Future<NIMResult<NIMChatroomQueueEntry>> pollChatroomQueueEntry(
      String roomId, String? key) {
    return _platform.pollChatroomQueueEntry(roomId, key);
  }

  /// 清空聊天室队列
  Future<NIMResult<void>> clearChatroomQueue(String roomId) {
    return _platform.clearChatroomQueue(roomId);
  }
}
